Package: DOtools
Title: Convenient functions to streamline your single cell data
        analysis workflow
Version: 1.1.0
Authors@R: 
    c(
    person(given = "Mariano",
           family = "Ruz Jurado",
           role = c("aut", "cre"),
           email = "ruzjurado@med.uni-frankfurt.de",
           comment = c(ORCID = "0000-0001-5354-5336")),
    person(given = "David",
           family = "Rodriguez Morales",
           email = "rodriguezmorales@med.uni-frankfurt.de",
           role = c("aut"),
           comment = c(ORCID = "0000-0002-1819-6991")),
    person(given = "David",
           family = "John",
           email = "john@med.uni-frankfurt.de",
           role = c("aut"),
           comment = c(ORCID = "0000-0003-3217-5449")),
    person(given = "DFG",
           family = "SFB 1366, Project B04",
           role = "fnd"),
    person(given = "DFG",
            family = "SFB 1531, Project 456687919",
            role = "fnd")       
    )
Description: This package provides functions for creating various visualizations, convenient wrappers, and quality-of-life utilities for single cell experiment objects. It offers a streamlined approach to visualize results and integrates different tools for easy use.
License: MIT + file LICENSE
BugReports: https://github.com/MarianoRuzJurado/DOtools/issues
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
biocViews: SingleCell, RNASeq, Visualization, Clustering, Annotation,
        WorkflowStep, QualityControl, GeneExpression
Depends: R (>= 4.5.0)
Imports: Seurat (>= 5.2.0), SeuratObject (>= 5.1.0), ggplot2 (>=
        3.5.0), ggpubr (>= 0.6.0), ggtext (>= 0.1.2), ggalluvial (>=
        0.12.5), tidyverse (>= 2.0.0), reshape2 (>= 1.4.4), dplyr (>=
        1.1.4), tidyr (>= 1.3.1), rstatix (>= 0.7.2), cowplot (>=
        1.1.3), reticulate (>= 1.41.0.1), zellkonverter (>= 1.16.0),
        progress (>= 1.2.3), ggiraphExtra (>= 0.3.0), grid (>= 4.4.3),
        SCpubr (>= 2.0.2), DropletUtils (>= 1.26.0), scCustomize (>=
        3.0.1), openxlsx (>= 4.2.8), tibble (>= 3.2.1), scDblFinder (>=
        1.20.0), ggcorrplot (>= 0.1.4.1), DESeq2 (>= 1.48.1), enrichR
        (>= 3.4), cli (>= 3.6.5), curl(>= 6.3.0), magrittr (>= 2.0.3),
        Matrix (>= 1.7.3), purrr(>= 1.0.4), rlang(>= 1.1.6), scales (>=
        1.4.0), SingleCellExperiment (>= 1.30.1), S4Vectors (>=
        0.46.0), basilisk (>= 1.20.0), methods, stats, utils
Suggests: SummarizedExperiment, knitr, kableExtra, pkgdown, RefManageR,
        BiocStyle, roxygen2, httr, magick, rmarkdown, assertthat, plyr,
        rsvg, scran, scater, igraph, sessioninfo, testthat (>= 3.0.0),
        mockery
VignetteBuilder: knitr
URL: https://marianoruzjurado.github.io/DOtools/
StagedInstall: yes
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/DOtools
git_branch: devel
git_last_commit: 527ea73
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 22:23:29 UTC; biocbuild
Author: Mariano Ruz Jurado [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5354-5336>),
  David Rodriguez Morales [aut] (ORCID:
    <https://orcid.org/0000-0002-1819-6991>),
  David John [aut] (ORCID: <https://orcid.org/0000-0003-3217-5449>),
  DFG SFB 1366, Project B04 [fnd],
  DFG SFB 1531, Project 456687919 [fnd]
Maintainer: Mariano Ruz Jurado <ruzjurado@med.uni-frankfurt.de>
