# Generated by roxygen2: do not edit by hand

export(DO.BarplotClustert)
export(DO.BarplotWilcox)
export(DO.BoxPlot)
export(DO.CellBender)
export(DO.CellComposition)
export(DO.CellTypist)
export(DO.Correlation)
export(DO.DietSCE)
export(DO.Dotplot)
export(DO.FullRecluster)
export(DO.Heatmap)
export(DO.Import)
export(DO.Integration)
export(DO.MultiDGE)
export(DO.PyEnv)
export(DO.SplitBarGSEA)
export(DO.Subset)
export(DO.TransferLabel)
export(DO.UMAP)
export(DO.VlnPlot)
export(DO.enrichR)
export(DO.scVI)
import(DESeq2)
import(DropletUtils)
import(Seurat)
import(SeuratObject)
import(SingleCellExperiment)
import(basilisk)
import(cli)
import(curl)
import(dplyr)
import(enrichR)
import(ggalluvial)
import(ggcorrplot)
import(ggplot2)
import(ggpubr)
import(ggtext)
import(magrittr)
import(openxlsx)
import(progress)
import(reshape2)
import(tibble)
import(tidyverse)
import(zellkonverter)
importFrom(SeuratObject,DefaultAssay)
importFrom(SeuratObject,as.Seurat)
importFrom(SingleCellExperiment,reducedDim)
importFrom(basilisk,basiliskRun)
