% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.example_10x}
\alias{.example_10x}
\title{Download example dataset 10x}
\usage{
.example_10x(base_dir = tempfile("dotools_datasets_"))
}
\value{
directory path where the data was saved
}
\description{
Download example dataset 10x
}
\keyword{internal}
